const {sores, soresWithAttach } = require('./util');

/**
 * Call SORES with operation `GetSorEntity`.
 * 
 * "Returnerer information for hver af de forespurgte SOR entiteter, der gjaldt på den angivne dato*"
 * 
 * @param {*} attach - The usual `attach`-function for Cucumber.
 * @param {string[]} sorIds - Array of string representing SOR IDs. E.g. `['8041000016000', '1331000016003']`.
 * @param {string?} [date] - An optional date in string format. E.g. `'2024-12-10'`, or `undefined` or `null` for no date.
 * @returns 
 */
async function GetSorEntity(attach, sorIds, date) {
    const body = {
        op: 'GetSorEntity',
        sorIds: sorIds,
        date: date,
    };

    return await soresWithAttach(attach, "", body);
}

/**
 * Call SORES with operation `SearchAllChildren`.
 * 
 * "Returnerer information for alle under-entiteter for hver af de forespurgte SOR entiteter, der gjaldt på den angivne dato*"
 * 
 * @param {*} attach - The usual `attach`-function for Cucumber.
 * @param {string[]} sorIds - Array of string representing SOR IDs. E.g. `['246061000016002', '1331000016003']`.
 * @param {string?} [date] - An optional date in string format. E.g. `'2024-12-10'`, or `undefined` or `null` for no date.
 * @returns 
 */
async function SearchAllChildren(attach, sorIds, date) {
    const body = {
        op: 'SearchAllChildren',
        sorIds: sorIds,
        date: date,
    };

    return await soresWithAttach(attach, "", body);
}

/**
 * Call SORES with operation `MapShakToSor`.
 * 
 * "Returnerer information for de SOR-entiteter, der på den angivne dato* havde det forespurgte SHAK id"
 * 
 * @param {*} attach - The usual `attach`-function for Cucumber.
 * @param {string} shakId - String representing SHAK ID. E.g. `'5501'`.
 * @param {string?} [date] - An optional date in string format. E.g. `'2024-12-10'`, or `undefined` or `null` for no date.
 * @returns 
 */
async function MapShakToSor(attach, shakId, date) {
    const body = {
        op: 'MapShakToSor',
        shakId: shakId,
        date: date,
    };

    return await soresWithAttach(attach, "", body);
}

/**
 * Call SORES with operation `GetSorEntityByProvider`.
 * 
 * "Returnerer information for (en liste af) alle SOR entiteter, der var tildelt det angivne ydernummer på den angivne dato*"
 * 
 * @param {*} attach - The usual `attach`-function for Cucumber.
 * @param {string[]} providerIds - Array of string representing ydernumre. E.g. `['303712', '101238']`.
 * @param {string?} [date] - An optional date in string format. E.g. `'2024-12-10'`, or `undefined` or `null` for no date.
 * @returns 
 */
async function GetSorEntityByProvider(attach, providerIds, date) {
    const body = {
        op: 'GetSorEntityByProvider',
        providerIds: providerIds,
        date: date,
    };

    return await soresWithAttach(attach, "", body);
}

/**
 * Call SORES with operation `IsSorEntityDescendantOfSorEntity`.
 * 
 * "Returnerer et array af booleans - for hvert par af id'er [id, mulig forælder] - om den pågældende enhed hørte under den mulige forælder i hierarkiet på den angivne dato*"
 * 
 * @param {*} attach - The usual `attach`-function for Cucumber.
 * @param {string[][]} sorIdAndParentSorIdPairs - 2D-array of strings representing SOR IDs. E.g. `[ ['246061000016002', '246081000016009'], ['246081000016009', '246061000016002'] ]`. Notice the inner arrays in the outer array.
 * @param {string?} [date] - An optional date in string format. E.g. `'2024-12-10'`, or `undefined` or `null` for no date.
 * @returns 
 */
async function IsSorEntityDescendantOfSorEntity(attach, sorIdAndParentSorIdPairs, date) {
    const body = {
        op: 'IsSorEntityDescendantOfSorEntity',
        sorIdAndParentSorIdPairs: sorIdAndParentSorIdPairs,
        date: date,
    };

    return await soresWithAttach(attach, "", body);
}

/**
 * Call SORES with operation `EntityNameLookup`.
 * 
 * "Returnerer information for et mindre antal SOR entiteter, hvis navn indeholder den angivne søgestreng begrænset af at entiteten:
 * 
 *  - ligger i SOR-træet under en entitet med det givne CVR-nummer 
 *  - har en entitets-type, der findes i den givne liste af entitets-typer
 *  - var gyldig på den angivne dato, eller dags dato, hvis ingen dato er angivet
 * 
 * Hvis intet CVR-nummer er angivet, søges i hele SOR-træet.  
 * Hvis entitets-type listen ikke er angivet eller er tom, returneres entiteter uanset deres type.  
 * Det anbefales på det kraftigste at anvende filtrering i videst mulige omfang.  
 * 
 * Såfremt søgningen matcher flere SOR entiteter end servicen er konfigureret til at returnere, indkluderer responset json-markøren `AdditionalRecordsExist` med værdien `True`."
 * 
 * @param {*} attach - The usual `attach`-function for Cucumber.
 * @param {string} matchEntityName - String to search for. E.g. `'stor'` when searching for 'Storkereden'.
 * @param {string?} cvr - Optional string - used to limit the search. E.g. `'34607079'`.
 * @param {string[]?} entityTypeNames - Optional array of strings - used to limit the search. E.g. `['jordemoderklinik']`.
 * @param {string?} [date] - An optional date in string format. E.g. `'2024-12-10'`, or `undefined` or `null` for no date.
 * @returns 
 */
async function EntityNameLookup(attach, matchEntityName, cvr, entityTypeNames, date) {
    const body = {
        op: 'EntityNameLookup',
        matchEntityName: matchEntityName,
        cvr: cvr,
        entityTypeNames: entityTypeNames,
        date: date,
    };

    return await soresWithAttach(attach, "", body);
}

/**
 * Call SORES with operation `InstitutionOwnerCvrNumber`.
 * 
 * "Returnerer CVR nummeret på den Institutionsejer som den forespurgte SOR enhed hørte under på den angivne dato*"
 * 
 * @param {*} attach - The usual `attach`-function for Cucumber.
 * @param {string[]} sorIds - Array of string representing SOR IDs. E.g. `['8041000016000', '1331000016003']`.
 * @param {string?} [date] - An optional date in string format. E.g. `'2024-12-10'`, or `undefined` or `null` for no date.
 * @returns 
 */
async function InstitutionOwnerCvrNumber(attach, sorIds, date) {
    const body = {
        op: 'InstitutionOwnerCvrNumber',
        sorIds: sorIds,
        date: date,
    };

    return await soresWithAttach(attach, "", body);
}

/**
 * Call SORES with operation `InstitutionOwnerCvrNumberByShak`.
 * 
 * "Returnerer CVR nummeret på den Institutionsejer som den forespurgte SHAK enhed hører under. Der tjekkes udelukkende på de fire første cifre af SHAK-id'et"
 * 
 * @param {*} attach - The usual `attach`-function for Cucumber.
 * @param {string[]} shakIds - Array of string representing SHAK IDs. E.g. `[ '1301', '6610123', '9999' ]`.
 * @returns 
 */
async function InstitutionOwnerCvrNumberByShak(attach, shakIds) {
    const body = {
        op: 'InstitutionOwnerCvrNumberByShak',
        shakIds: shakIds,
    };

    return await soresWithAttach(attach, "", body);
}

module.exports = {
    GetSorEntity,
    SearchAllChildren,
    MapShakToSor,
    GetSorEntityByProvider,
    IsSorEntityDescendantOfSorEntity,
    EntityNameLookup,
    InstitutionOwnerCvrNumber,
    InstitutionOwnerCvrNumberByShak,
};
